Package: DegNorm
Type: Package
Title: DegNorm: degradation normalization for RNA-seq data
Version: 1.4.0
Date: 2020-09-22
Author: Bin Xiong and Ji-Ping Wang
Maintainer: Ji-Ping Wang <jzwang@northwestern.edu>
biocViews: RNASeq, Normalization, GeneExpression, Alignment,Coverage,
        DifferentialExpression, BatchEffect,Software,Sequencing,
        ImmunoOncology, QualityControl, DataImport
Description: This package performs degradation normalization in bulk RNA-seq data to improve differential expression analysis accuracy.
License: LGPL (>= 3)
Depends: R (>= 4.0.0), methods
Imports: Rcpp (>= 1.0.2),GenomicFeatures, parallel, foreach, S4Vectors,
        doParallel, Rsamtools (>= 1.31.2), GenomicAlignments,
        heatmaply, data.table, stats, ggplot2, GenomicRanges, IRanges,
        plyr, plotly, utils,viridis
LinkingTo: Rcpp, RcppArmadillo,S4Vectors,IRanges
NeedsCompilation: yes
Packaged: 2021-10-26 21:19:23 UTC; biocbuild
Suggests: knitr,rmarkdown,formatR
VignetteBuilder: knitr
BugReports: https://github.com/jipingw/DegNorm/issues
git_url: https://git.bioconductor.org/packages/DegNorm
git_branch: RELEASE_3_14
git_last_commit: a1776e6
git_last_commit_date: 2021-10-26
Date/Publication: 2021-10-26

Package: DegNorm
Type: Package
Title: DegNorm: degradation normalization for RNA-seq data
Version: 1.22.0
Date: 2025-08-11
Authors@R: 
    person("bin", "Xiong",
        email = "binxiong2012@u.northwestern.edu",
        role = c("aut"))
    person("Ji-Ping", "Wang",
        email = "jzwang@northwestern.edu",
        role = c("aut", "cre"),
        comment = c(ORCID="0000-0002-8398-939X"))
biocViews: RNASeq, Normalization, GeneExpression, Alignment,Coverage,
        DifferentialExpression, BatchEffect,Software,Sequencing,
        ImmunoOncology, QualityControl, DataImport
Description: This package performs degradation normalization in bulk RNA-seq data to improve differential expression analysis accuracy. It provides estimates for each gene within each sample.
License: LGPL (>= 3)
Depends: R (>= 4.0.0), methods
Imports: Rcpp (>= 1.0.2),GenomicFeatures, txdbmaker, parallel, foreach,
        S4Vectors, doParallel, Rsamtools (>= 1.31.2),
        GenomicAlignments, heatmaply, data.table, stats, ggplot2,
        GenomicRanges, IRanges, plyr, plotly, utils,viridis
LinkingTo: Rcpp, RcppArmadillo,S4Vectors,IRanges
NeedsCompilation: yes
Packaged: 2026-04-28 21:56:58 UTC; biocbuild
Suggests: knitr,rmarkdown,formatR
VignetteBuilder: knitr
BugReports: https://github.com/jipingw/DegNorm/issues
git_url: https://git.bioconductor.org/packages/DegNorm
git_branch: RELEASE_3_23
git_last_commit: da6503c
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.23
Date/Publication: 2026-04-28
Author: Ji-Ping Wang [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-8398-939X>)
Maintainer: Ji-Ping Wang <jzwang@northwestern.edu>
Built: R 4.6.0; aarch64-apple-darwin23; 2026-04-29 04:39:53 UTC; unix

Package: DegNorm
Type: Package
Title: DegNorm: degradation normalization for RNA-seq data
Version: 1.16.0
Date: 2024-03-26
Author: Bin Xiong and Ji-Ping Wang
Maintainer: Ji-Ping Wang <jzwang@northwestern.edu>
biocViews: RNASeq, Normalization, GeneExpression, Alignment,Coverage,
        DifferentialExpression, BatchEffect,Software,Sequencing,
        ImmunoOncology, QualityControl, DataImport
Description: This package performs degradation normalization in bulk RNA-seq data to improve differential expression analysis accuracy.
License: LGPL (>= 3)
Depends: R (>= 4.0.0), methods
Imports: Rcpp (>= 1.0.2),GenomicFeatures, txdbmaker, parallel, foreach,
        S4Vectors, doParallel, Rsamtools (>= 1.31.2),
        GenomicAlignments, heatmaply, data.table, stats, ggplot2,
        GenomicRanges, IRanges, plyr, plotly, utils,viridis
LinkingTo: Rcpp, RcppArmadillo,S4Vectors,IRanges
NeedsCompilation: yes
Packaged: 2024-10-30 00:24:15 UTC; biocbuild
Suggests: knitr,rmarkdown,formatR
VignetteBuilder: knitr
BugReports: https://github.com/jipingw/DegNorm/issues
git_url: https://git.bioconductor.org/packages/DegNorm
git_branch: RELEASE_3_20
git_last_commit: c2580b9
git_last_commit_date: 2024-10-29
Repository: Bioconductor 3.20
Date/Publication: 2024-10-29

Package: DegNorm
Type: Package
Title: DegNorm: degradation normalization for RNA-seq data
Version: 1.22.0
Date: 2025-08-11
Authors@R: 
    person("bin", "Xiong",
        email = "binxiong2012@u.northwestern.edu",
        role = c("aut"))
    person("Ji-Ping", "Wang",
        email = "jzwang@northwestern.edu",
        role = c("aut", "cre"),
        comment = c(ORCID="0000-0002-8398-939X"))
biocViews: RNASeq, Normalization, GeneExpression, Alignment,Coverage,
        DifferentialExpression, BatchEffect,Software,Sequencing,
        ImmunoOncology, QualityControl, DataImport
Description: This package performs degradation normalization in bulk RNA-seq data to improve differential expression analysis accuracy. It provides estimates for each gene within each sample.
License: LGPL (>= 3)
Depends: R (>= 4.0.0), methods
Imports: Rcpp (>= 1.0.2),GenomicFeatures, txdbmaker, parallel, foreach,
        S4Vectors, doParallel, Rsamtools (>= 1.31.2),
        GenomicAlignments, heatmaply, data.table, stats, ggplot2,
        GenomicRanges, IRanges, plyr, plotly, utils,viridis
LinkingTo: Rcpp, RcppArmadillo,S4Vectors,IRanges
NeedsCompilation: yes
Packaged: 2026-04-28 21:43:24 UTC; biocbuild
Suggests: knitr,rmarkdown,formatR
VignetteBuilder: knitr
BugReports: https://github.com/jipingw/DegNorm/issues
git_url: https://git.bioconductor.org/packages/DegNorm
git_branch: RELEASE_3_23
git_last_commit: da6503c
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.23
Date/Publication: 2026-04-28
Author: Ji-Ping Wang [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-8398-939X>)
Maintainer: Ji-Ping Wang <jzwang@northwestern.edu>

useDynLib(DegNorm, .registration=TRUE)
importFrom(Rcpp, evalCpp)
import(methods)
importFrom(GenomicAlignments,GAlignments,readGAlignmentPairs,readGAlignments)
importFrom(GenomicFeatures,pcoverageByTranscript,exonsBy)
importFrom(txdbmaker,makeTxDbFromGFF)
importFrom(Rsamtools,ScanBamParam,BamFile,indexBam,scanBamFlag,testPairedEndBam)
importFrom(IRanges,RleList, extractList,IntegerList,stack)
importFrom(heatmaply,heatmaply)
importFrom(GenomicRanges,findOverlaps,reduce,seqinfo,seqnames,strand)
importFrom(S4Vectors, countQueryHits, endoapply, queryHits, runValue)
importFrom(ggplot2,aes, facet_grid, geom_boxplot, geom_line, ggplot, 
    stat_boxplot, theme,theme_classic, theme_light, xlab, ylab, ylim,
    scale_color_manual)
importFrom(doParallel,registerDoParallel,stopImplicitCluster)
importFrom(foreach, "%dopar%", foreach)
importFrom(parallel, detectCores,makeCluster,stopCluster)
importFrom(plotly,plot_ly,subplot)
importFrom(data.table, data.table)
importFrom(stats, setNames,cor,median)
importFrom(utils, globalVariables)
importFrom(plyr,mutate)
import(viridis)
## Export functions
export(degnorm, 
    read_coverage_batch,
    summary_CoverageClass,
    summary_DegNormClass,
    plot_heatmap,
    plot_corr,
    plot_coverage,
    plot_boxplot)

DegNorm R package
================

**Maintainer**: Ji-Ping Wang, \<<jzwang@northwestern.edu>\>

([**Python package
download**](https://nustatbioinfo.github.io/DegNorm/))\*\*

**Reference**: Xiong, B., Yang, Y., Fineis, F. Wang, J.-P., DegNorm:
normalization of generalized transcript degradation improves accuracy in
RNA-seq analysis, Genome Biology, 2019,20:75

## What is DegNorm?

DegNorm, short for **Deg**radation **Norm**alization, is a
bioinformatics pipeline designed to correct for bias due to the
heterogeneous patterns of transcript degradation in RNA-seq data.
DegNorm helps improve the accuracy of the differential expression
analysis by accounting for this degradation.

In practice, RNA samples are often more-or-less degraded, and the
degradation severity is not only sample-specific, but gene-specific as
well. It is known that longer genes tend to degrade faster than shorter
ones. As such, commonplace global degradation normalization approaches
that impose a single normalization factor on all genes within a sample
can be ineffective in correcting for RNA degradation bias.

## DegNorm pipline available formats

We’ve developed an R package and an indepedent Python package
([download](https://nustatbioinfo.github.io/DegNorm/)), both of which
allow to run the entire pipeline from the RNA-seq alignment (.bam)
files.

## DegNorm main features

DegNorm R package contains two major functions: (1) processing the
RNA-seq alignment file (.bam) to calculate the coverage; and (2) using a
core algorithm written in RcppArmadillo to perform rank-one
over-approximation on converage matrices for each gene to estimate the
degramation index (DI) score for each gene within each sample.

DegNorm outputs DI scores together with degradation-normalized read
counts (based on DI scores). It also provides supplementary functions
for visualization of degradation at both gene and sample level. The
following diagram illustrates the flow of DegNorm pipeline.

 

![A diagram of
DegNorm.](http://bioinfo.stats.northwestern.edu/~jzwang/DegNorm/degnorm_logo.png)

 

The following vignette is intended to provide example codes for running
DegNorm R package. It presumes that you have successfully installed
DegNorm package. We illustrate below how to: 1) calculate the read
coverage curves for all genes within all samples, and 2) perform
degradation normalization on coverage curves. Either step is computing
intensive. Dependent upon the number of samples and the sequencing
depth, the total computing time may last a few hours. DegNorm utilizes
the parallel computing functionality of R and automatically detects the
number of cores on your computer to run jobs in parallel. Due to the
large size of bam file and limited computing power of personal computer,
we recommend users to run it in servers or computing
clusters.

### 1\. Compute coverage score based on alignment .bam files

##### Set up input file: .bam and .gtf files.

``` r
## specify bam_files from RNA-seq, you should replace it by your own bam files
bam_file_list=list.files(path=system.file("extdata",package="DegNorm"),
                        pattern=".bam$",full.names=TRUE)
```

The three bam files were subsetted from a specific region of chorosome
21 from the origianl bam for package size limitation. Original files can
be found from the included reference
above.

``` r
## gtf_file you used for RNA-seq alignment, replace it by your own gtf file
gtf_file=list.files(path=system.file("extdata",package="DegNorm"),
                    pattern=".gtf$",full.names=TRUE)
```

##### Run main function to create read coverage matrix and read counts

``` r
## calculate the read coverage score for all genes of all samples
coverage_res_chr21_sub=read_coverage_batch(bam_file_list, gtf_file,cores=2)
```

    ## Parse gtf file...done 
    ## Index SRR873822_chr21_9900000-10000000.bam 
    ## Computing coverage for SRR873822_chr21_9900000-10000000.bam ... 
    ##    SRR873822_chr21_9900000-10000000.bam is a paired-end bam file 
    ## SRR873822_chr21_9900000-10000000.bam is done! 
    ## 
    ## Index SRR873834_chr21_9900000-10000000.bam 
    ## Computing coverage for SRR873834_chr21_9900000-10000000.bam ... 
    ##    SRR873834_chr21_9900000-10000000.bam is a paired-end bam file 
    ## SRR873834_chr21_9900000-10000000.bam is done! 
    ## 
    ## Index SRR873838_chr21_9900000-10000000.bam 
    ## Computing coverage for SRR873838_chr21_9900000-10000000.bam ... 
    ##    SRR873838_chr21_9900000-10000000.bam is a paired-end bam file 
    ## SRR873838_chr21_9900000-10000000.bam is done!

`cores` argument specifies the number of cores to use. Users should try
to use as many as possible cores to maximize the computing efficiency.

``` r
## save the coverage results
save(coverage_res_chr21_sub,file="coverage_res_chr21_sub.Rda")
```

Function `read_coverage_batch` returns the coverage matrices as a list,
one per gene, and a dataframe for read counts, each row for one gene and
each column for one sample.

``` r
data("coverage_res_chr21")
## summarize the coverage results
summary_CoverageClass(coverage_res_chr21)
```

    ## CoverageClass from read_coverage_batch function 
    ## $coverage    : list of length 339 
    ## $counts  : data.frame of dimension 339 by 3 
    ## 
    ## Samples:      SRR873822_chr21.bam SRR873834_chr21.bam SRR873838_chr21.bam 
    ## Total number genes:   339

``` r
## extract coverage scores and counts from coverage_res
coverage_matrix=coverage_res_chr21$coverage
counts=coverage_res_chr21$counts
```

### 2\. DegNorm core algorithm

Run degnorm core algorithm for degradation normalization. DegNorm
purpose is for differential expression analysis. Thus genes with
extremely low read counts from all samples are filtered out. The current
filtering criterion is that if more than half of the samples have less
than 5 read count, that gene will not be considered in the degnorm
algorithm. In the following example, I am using downsamling to save time
below (default). Alternatively you can set down\_sampling = 0, which
takes longer time. If `down_samplin= 1`, read coverage scores are binned
with size by `grid_size` for baseline selection to achieve better
efficiency. The default `grid_size` is 10 bp. We recommend to use a
`grid_size` less than 50 bp. `iteration` specifies the big loop in
DegNorm algorithm and 5 is usually sufficient. `loop` specifies the
iteration number in the matrix factorization over-approximation.

``` r
res_DegNorm_chr21 = degnorm(read_coverage = coverage_res_chr21[[1]],
                    counts = coverage_res_chr21[[2]],
                    iteration = 5,
                    down_sampling = 1,
                    grid_size=10,
                    loop = 100,
                    cores=2)
```

If `down_sampling= 0`, then the argument `grid_size` is ignored.

``` r
## save the DegNorm results
save(res_DegNorm_chr21,file="res_DegNorm_chr21.Rda")
```

Function `degnorm` returns a list of multiple objects. counts\_normed is
the one with degradation normalized read counts for you to input DeSeq
or EdgeR for DE analysis.

``` r
data("res_DegNorm_chr21")
```

``` r
## summary of the DegNorm output
summary_DegNormClass(res_DegNorm_chr21)
```

    ## DegNormClass from DegNorm function: 
    ## $counts  : data.frame of dimension 132 by 3 
    ## $counts_normed   : data.frame of dimension 132 by 3 
    ## $DI      : matrix array of dimension 132 by 3 
    ## $K       : matrix array of dimension 132 by 3 
    ## $convergence     : integer of length 132 
    ## $envelop     : list of length 132 
    ## 
    ## Samples:      SRR873822_chr21.bam SRR873834_chr21.bam SRR873838_chr21.bam 
    ## Total number genes:   132

The difference of number of genes between `res_DegNorm` and
`coverage_res` is 207 (339-132). The 207 genes were filtered out from
`degnorm` degradation normalization because less than half of the
samples (3) have more than 5 read count.

``` r
## extrac normalized read counts
counts_normed=res_DegNorm_chr21$counts_normed
```

### 3\. Plot functions in DegNorm

DegNorm provides four plot functions for visualization of degradation
and sample quality diagnosis.

  - plot\_coverage
  - plot\_corr
  - plot\_heatmap
  - plot\_boxplot

##### – Plot the before-/after-degradation coverage curves

``` r
##gene named "SOD1"
plot_coverage(gene_name="SOD1", coverage_output=coverage_res_chr21, 
            degnorm_output=res_DegNorm_chr21,group=c(0,0,1))
```

##### – Boxplot of the degradation index(DI) scores

``` r
plot_boxplot(DI=res_DegNorm_chr21$DI)
```

##### – Heatmap plot of the degradation index(DI) scores

``` r
plot_heatmap(DI=res_DegNorm_chr21$DI)
```

#### – Correlation matrix plot of degradation index(DI) scores

``` r
plot_corr(DI=res_DegNorm_chr21$DI)
```

################################################################################
# ratioSVD: function to calulate SVD for the raw coverage
#           used to identify genes with similar coverage pattern
#           across all samples (minimal degradation)
# input:    coverage matrix (f) with rows representing samples
# output:   DI scores (rho)
################################################################################

.ratioSVD <- function(f){
    model.svd = svd(f, nv = 1, nu = 1)
    fitted = model.svd$d[1] * model.svd$u %*% t(model.svd$v)
    fitted[fitted < f] = f[fitted < f]
    return(1-rowSums(f)/(rowSums(fitted)+1))
}

################################################################################
# degnorm: function to perform degradation normalization
# input:
#    - coverage: list of gene coverages, each row stands for one sample;
#    - counts: read count matrix with gene ordered same as the coverage list;
#    - iterations: number of iterations for DegNorm (default to be 5);
#    - loop: number of iterations for NMF-OA (default to be 100).
#    - down_sampling: 1 for yes and 0 or no. If yes, coverage score is averaged
#                     on an interval of size specificed by grid_size. The
#                   size-reduced coverage matrix is fed for baseline selection
#                     for efficiency consideration
#    - grid_size: the bin size for average coverage score
#    - cores: number of cores to use. Default is  maximum number cores -1
# output: a list containing the following
#    - DI: Degradation Index matrix;
#    - counts_normed: DegNorm normalized counts;
#    - convergence: convergence status indicating whether baseline is found.
#    - envelop: envelop curve which stands for the estimated commons
#               shape before degradation.
################################################################################
degnorm <- function(read_coverage, counts, iteration=5, loop=100,
                    down_sampling=1, grid_size=10,cores=1){
    # filtering out genes with low expressions
    if(length(read_coverage)!=nrow(counts)) stop("Error: number of genes from
                    coverage list is inconsistent with the counts matrix!")
    if((down_sampling %in% c(0,1))==FALSE) stop("Error: down_sampling should be
                    1 (down-sampling) or 0 (not down-sampling)!")
    if(grid_size %% 1!=0||grid_size<0) {
        stop("Error: grid_size must be a positive integer.")}
    # filtering out genes with low expressions
    message("Filtering out genes with low read counts (x<5)..")
    n = dim(counts)[2]
    filter = apply(counts, 1, function(x) length(x[x>5])>=floor(n/2))
    counts = counts[filter,];  m = dim(counts)[1]
    read_coverage = read_coverage[which(filter=="TRUE")]
    # Estimating library sizes / scaling factors from SVD
    message("Initial estimation of count normalization factors...")
    norm.factor = colSums(counts)/median(colSums(counts))
    # Register multiple nodes
    max.cores=detectCores(logical = FALSE)
    if(max.cores-1 < cores) cores <- max.cores-1
    cl <- makeCluster(cores);  registerDoParallel(cl)
    message("Initial degradation normalization without base-line selection...")
    rho = NULL
    i=1
    rho = foreach(i = seq_len(m),.combine = "rbind",
                    .export = ".ratioSVD") %dopar% {
                    .ratioSVD(f = read_coverage[[i]])}
    stopImplicitCluster()
    stopCluster(cl)
    norm.factor = colSums(counts[apply(rho,1,max) < 0.10,])
    if(sum(norm.factor)>0){
        baseline=1
    }else{
        norm.factor=apply(counts,2,sum)
        baseline=0
    }
    norm.factor = norm.factor/stats::median(norm.factor)
    scale = norm.factor
    counts.weighted = sweep(counts, 2, norm.factor, FUN = "/")
    message("DegNorm core algorithm starts...")
    # Calculating Degradation Index
    output=.degnorm_baseline(read_coverage, counts, iteration, loop,
                            down_sampling, grid_size,scale,counts.weighted,
                            cores, baseline)
    message("DegNorm core algorithm done!")
    class(output)="DegNormClass"
    return(output)
}

##degnorm baseline selection algorithm
.degnorm_baseline=function(read_coverage, counts, iteration, loop,
                        down_sampling, grid_size,scale,counts.weighted,cores,
                        baseline=1){
    for(i in seq_len(iteration)){
        cat(paste0("   DegNorm iteration ",i), "\n")
        #cores <- detectCores(logical = FALSE) #utilize multiple cores
        cl <- makeCluster(cores)
        #cl <- makeCluster(cores-1)
        registerDoParallel(cl)
        res = NULL;j=1;m=dim(counts)[1];n=dim(counts)[2]
        if(down_sampling==0){
            res = foreach(j = seq_len(m), .multicombine = TRUE,
                .export = c(".optiNMFCPP",".NMFCPP",".bin_drop")) %dopar%{
                .optiNMFCPP(read_coverage[[j]], scale, loop, baseline)}
        }else if (down_sampling==1){
            res = foreach(j = seq_len(m), .multicombine = TRUE,
                .export = c(".optiNMFCPP_grid",".NMFCPP",".bin_drop")) %dopar%{
                .optiNMFCPP_grid(read_coverage[[j]], scale, loop, grid_size)}
        }else{
            stop("Error:down_sampling argument should either 0 or 1!")
        }
        stopImplicitCluster()
        stopCluster(cl)
        rho = t(do.call(cbind, lapply(res, function(x) x$rho)))
        # in case rho contains NA, replace entire row by 0
        rho[apply(rho,1,function(x) sum(is.na(x))>0),]=0
        K = t(do.call(cbind, lapply(res, function(x) x$K)))
        convergence = vapply(res, function(x) x$convergence,numeric(1))
        envelop= lapply(res, function(x) x$envelop)
        dimnames(rho)=dimnames(counts);dimnames(K)=dimnames(counts);
        names(convergence)=row.names(counts); names(envelop)=row.names(counts)
        # Capping the DI scores at 0.9 to adjust for over-inflation
        rho[rho > 0.9] = 0.9; rho[rho < 0] = 0
        # Getting iterative DegNorm normalized counts
        counts_normed = counts.weighted / (1 - rho)
        # Applying sample-wise overall DI scores to low counts genes
        # which did not go through NMF
        rho[apply(rho,1,max)==0,] =
        matrix(rep(1-colSums(counts.weighted)/colSums(counts_normed),
                        sum(apply(rho,1,max)==0)),byrow = TRUE, ncol = n)
        # Updating sequencing depth scale
        norm.factor = colSums(counts_normed)
        norm.factor = norm.factor / median(norm.factor)
        counts.weighted = sweep(counts.weighted, 2, norm.factor, FUN = "/")
        scale = scale * norm.factor
    }
    output = list("counts"=counts,"counts_normed"=round(counts_normed),
                    "DI"=rho,"K"=K,"convergence"=convergence,
                    "envelop"=envelop)
    return(output)
}

###############################################################################
############# Coverage calculation for batch files ############################
## read_coverage_batch: function to calculate read coverage score in batches
##    input:
##       - bam_file_list: list of bam file names
##       - gtf_file: name of gtf file where the RNA-seq data were aligned with 
##                 reference to.
##       - cores: number of cores to use.
##    output:
##       - coverageClass object, which includes:
##         (1) list of coverage matrices 
##         (2) a dataframe for read counts          
##
## Notes: 
##   1. Coverage score is calcualted per gene, i.e. concatenation of all
##      exons from the same gene.
##   2. We follow HTseq for counting valid read/read pairs for each gene.
##   3. When reading alignment file, isSecondaryAlignment flag is set as 
##      FALSE to avoid possible redundant counting.
##   4. For paired-end data, isPaired is set as TRUE. We don't recommend
##      setting isProperPair as TRUE as some fragments length may exceed 200bp.
##   5. User can modify scanBamParam in the R codes below as needed.
###############################################################################
read_coverage_batch=function(bam_file_list,gtf_file,cores=1){
    i=1; options(warn=-1)
    all_genes=.gtf_parse(gtf_file)
    ##simplify bam file names if path information present in file names
    sample_names=as.vector(vapply(bam_file_list, function(x) 
                        rev(strsplit(x,"[\\\\]|[/]")[[1]])[1],character(1)))
    for (k in seq_along(bam_file_list)){
        #if bam index not exist, generate index
        if(!file.exists(paste(sample_names[k],".bai",sep=""))){
            cat("Index", sample_names[k],"\n")
            indexBam(bam_file_list[k])
        }
        cat ("Computing coverage for", sample_names[k],"...", "\n")
        max.cores=detectCores(logical = FALSE)
        if(max.cores-1<cores) cores <- max.cores-1
        results=read_coverage(bam_file_list[k], all_genes,cores)
        ##separate coverage and read_counts
        coverage_RLE=RleList()    ## coverage in Rle format
        read_counts=data.frame()  ## data frame for read-counts
        read_counts=do.call(rbind,lapply(results,`[[`,2))
        coverage_RLE=lapply(results,`[[`,1)
        ##convert Rle object into vector coverage
        cl <- makeCluster(cores)
        registerDoParallel(cl)
        coverage_RLE=coverage_RLE[lengths(coverage_RLE)>0]
        coverage_vector=foreach(i = seq_along(coverage_RLE),
                .packages=c("Rsamtools","GenomicAlignments","GenomicFeatures"),
                            .inorder=TRUE,.multicombine=TRUE) %dopar%
                    {  res=lapply(coverage_RLE[[i]],as.vector)
                        return(res)}
        stopImplicitCluster()
        stopCluster(cl)
        coverage_vector=do.call(c,coverage_vector)
        cat (sample_names[k], "is done!", "\n\n")
        if (k==1){
                coverage_matrix=coverage_vector
                read_counts_matrix=read_counts
        }else {
                coverage_matrix=mapply(rbind,coverage_matrix,coverage_vector)
                read_counts_matrix=cbind(read_counts_matrix,read_counts)
        }   
        coverage_results=list(coverage_matrix,read_counts_matrix)
    }
    colnames(read_counts_matrix)=sample_names
    coverage_matrix=lapply(coverage_matrix,function(x) 
                            {row.names(x)=sample_names; x})
    output=list(coverage=coverage_matrix,counts=read_counts_matrix)
    class(output)="CoverageClass"
    return(output)
}

################################gtf file parsing ###############################
# .gtf_parse: function to parse gtf file by compiling all exons from the same
#            gene to generate a total transcript. Genes which mapped to multiple
#             chromosomes or multiple strands are excluded to avoid confusion.
##    input:
##       - gtf_file: name of gtf file where the RNA-seq data were aligned with 
##                  reference to.
##    ouput:
##       - GRangesList object
################################################################################
.gtf_parse<-function(gtf_file){
    ## parse gtf file to create GRangesList object
    cat("Parse gtf file...") ## parse gtf file to create GRangesList object
    suppressMessages(txdb <- makeTxDbFromGFF(gtf_file))
    all_genes <- exonsBy(txdb, by="gene")
    ##exclude genes on two strands or genes exist on multiple chromosomes
    all_genes=all_genes[vapply(unique(runValue(strand(all_genes))),
                            length,numeric(1))==1]
    all_genes=all_genes[vapply(unique(runValue(seqnames(all_genes))),
                            length,numeric(1))==1]
    ##sorting exons on - strand into ascending rank
    all_genes=endoapply(all_genes, function(x) if(runValue(strand(x))=="-") 
                {return(rev(x))}else{return(x)})
    all_genes <- IRanges::reduce(all_genes)
    cat("done", "\n")
    return(all_genes)
}



################################################################################
######### Converage calculation for one bam file ###############################
## read_coverage: function to calculate read coverage score for one bam file
##                This function determines whether it single-end and paired-end,
##                and generates bam file index if needed.It takes multiple-core 
##                structure for parallel computing for efficiency.
################################################################################
##    input:
##       - bam_file: bam file names
##       - all_genes: parsed genes.gtf file
##       - cores: number of cores
##
##    output:
##       - coverageClass object, which includes:
##         (1) list of coverage score for each gene in RLE format and 
##         (2) a dataframe for read counts          
##
################################################################################


read_coverage=function(bam_file,all_genes,cores){
    cl <- makeCluster(cores)
    registerDoParallel(cl)

    ##Filter out empty chromosomes (i.e. no genes  on chromosomes)
    ##chrom=seqnames(seqinfo(all_genes))
    chrom=as.vector(unique(unlist(runValue(seqnames(all_genes)))))

    ##for memory issue, coverage is calculated by chromosomes
    sample_name=rev(strsplit(bam_file,"[\\\\]|[/]")[[1]])[1]
    i=1
    if(suppressMessages(testPairedEndBam(bam_file))==TRUE){
        cat("  ",sample_name, "is a paired-end bam file","\n")
        results <- foreach(i = seq_along(chrom),
                        .export=c(".paired_end_cov_by_ch",
                        ".single_end_cov_by_ch", ".IntersectionStrict2"),
                        .packages = c("Rsamtools", "GenomicAlignments",
                                    "GenomicFeatures","data.table"),
                        .inorder=TRUE,.multicombine=TRUE) %dopar%
                        { 
                        res=.paired_end_cov_by_ch(bam_file,all_genes,chrom[i])
                        return(res)
                        }
    }else{
        cat("  ",sample_name, "is a single-end bam file","\n")
        results <- foreach(i = seq_along(chrom),
                        .export=c(".paired_end_cov_by_ch",
                                ".single_end_cov_by_ch",".IntersectionStrict2"),
                        .packages = c("Rsamtools", "GenomicAlignments",
                                "GenomicFeatures"),
                        .inorder=TRUE,.multicombine=TRUE) %dopar%
                        { 
                        res=.single_end_cov_by_ch(bam_file,all_genes,chrom[i])
                        return(res)}
    }
    stopImplicitCluster()
    stopCluster(cl)
    return(results)
}

################################################################################
###### Coverage calculation for paired end for one chromosome###################
## .paired_end_cov_by_ch: function to calculate read coverage score for one
##  chromosome for paired-end data.
################################################################################
##    input:
##       - bam: bam file names
##       - all_genes: parsed genes.gtf file
##       - ch: chromosome name
##
##    output:
##       - coverageClass object, which includes:
##         (1) list of coverage score for each gene in RLE format and 
##         (2) a dataframe for read counts          
##
## Notes: 
##   1. Coverage score is calcualted per gene, i.e. concatenation of all
##      exons from the same gene.
##   2. We follow HTseq protocol to count valid reads/read pairs for each gene.
##   3. When reading alignment file, isSecondaryAlignment flag is set as 
##      FALSE to avoid possible redundant counting.
##   4. isPaired is set as TRUE. We don't recommend setting isProperPair 
##      as TRUE as some fragments length may exceed 200bp.
##   5. User can modify scanBamParam in the R codes below as needed.
##############################################################################

.paired_end_cov_by_ch=function(bam,all_genes,ch){
    ##function to calculate the coverage score and return in Rle objects
    bf = BamFile(bam, asMates = TRUE, qnameSuffixStart = ".")
    tx_compat_cvg=RleList()
    genes=all_genes[unlist(runValue(seqnames(all_genes)))==ch]
    gr = as(seqinfo(bf), "GRanges")
    if(ch %in% runValue(seqnames(gr))){
        param = ScanBamParam(which = gr[ch],flag=scanBamFlag(isPaired=TRUE, 
                            isSecondaryAlignment=FALSE))
        gal2=readGAlignmentPairs(bf, param = param)
        gal2=gal2[strand(gal2)!="*"]
        gal2=gal2[is.na(seqnames(gal2))==FALSE]
        grl <- as(gal2, "GRangesList")
        gal3=reduce(grl)
        #only keep genes on the current chromosome
        results=.IntersectionStrict2(genes,gal2)
        tx2reads <- setNames(as(t(results), "List"), names(genes))
        ## calculate read pair
        read_counts=data.frame(count=lengths(tx2reads))
        rownames(read_counts)=names(tx2reads)

        #relist gal3 based on compatibility
        temp0=data.table(group=as(gal3,"data.frame")$group,
                        index=seq_len(sum(lengths(gal3))))
        #group2indexMap=unique(setDT(temp0))[, .(values = index), by =group ] 

        idmatch=function(tx2reads,group2indexMap){
            return(group2indexMap[group2indexMap$group %in% tx2reads]$index)
        }
        #tx2reads1=lapply(tx2reads,idmatch,group2indexMap=group2indexMap)
        tx2reads1=lapply(tx2reads,idmatch,group2indexMap=temp0)
        gal4=unlist(gal3)
        rm(gal3)
        compat_reads_by_tx <- extractList(gal4, tx2reads1)
        tx_compat_cvg <- pcoverageByTranscript(compat_reads_by_tx,
                                            genes,
                                            ignore.strand=TRUE)
    }else{
        tx2reads1=IntegerList(vector("list", length(genes)))
        left_gal=GAlignments() #empty alignment file
        names(tx2reads1)=names(genes)
        read_counts=data.frame(count=lengths(tx2reads1))
        rownames(read_counts)=names(tx2reads1)
        compat_reads_by_tx <- extractList(left_gal, tx2reads1)
        tx_compat_cvg <- pcoverageByTranscript(compat_reads_by_tx,
                                            genes,
                                            ignore.strand=FALSE)
    }
    return(list(tx_compat_cvg,read_counts))
}

################################################################################
###### Coverage calculation for single end for one chromosome###################
## .single_end_cov_by_ch: function to calculate read coverage score for
##                       one chromosome for sigle-end data
################################################################################
##    input:
##       - bam: bam file names
##       - all_genes: parsed genes.gtf file
##       - ch: chromosome name
##
##    output:
##       - coverageClass object, which includes:
##         (1) list of coverage score for each gene in RLE format and 
##         (2) a dataframe for read counts          
##
## Notes: 
##   1. Coverage score is calcualted per gene, i.e. concatenation of all
##      exons from the same gene.
##   2. We follow HTseq protocol to count valid reads/read pairs for each gene.
##   3. When reading alignment file, isSecondaryAlignment flag is set as 
##      FALSE to avoid possible redundant counting.
##   4. User can modify scanBamParam in the R codes below as needed.
################################################################################
.single_end_cov_by_ch=function(bam,all_genes,ch){
    ##function to calculate the coverage score and return in Rle objects
    bf = BamFile(bam)
    tx_compat_cvg=RleList()
    read_counts=data.frame()

    genes=all_genes[unlist(runValue(seqnames(all_genes)))==ch]
    gr = as(seqinfo(bf), "GRanges")
    if(ch %in% runValue(seqnames(gr))){
        param = ScanBamParam(which = gr[ch],
                            flag=scanBamFlag(isSecondaryAlignment=FALSE))
        gal2=readGAlignments(bf, param = param)
        gal2=gal2[strand(gal2)!="*"]
        gal2=gal2[is.na(seqnames(gal2))==FALSE]

        #only keep genes on the current chromosome
        results=.IntersectionStrict2(genes,gal2)
        tx2reads <- setNames(as(t(results), "List"), names(genes))
        compat_reads_by_tx <- extractList(gal2, tx2reads)

        read_counts=data.frame(count=lengths(tx2reads))
        rownames(read_counts)=names(tx2reads)
        tx_compat_cvg <- pcoverageByTranscript(compat_reads_by_tx,
                                            genes,
                                            ignore.strand=TRUE)
    }else{
        tx2reads1=IntegerList(vector("list", length(genes)))
        names(tx2reads1)=names(genes)
        left_gal=GAlignments()
        compat_reads_by_tx <- extractList(left_gal, tx2reads1)
        read_counts=data.frame(count=lengths(tx2reads1))
        colnames(read_counts)=names(tx2reads1)  

        tx_compat_cvg <- pcoverageByTranscript(compat_reads_by_tx,
                                            genes,
                                            ignore.strand=FALSE)
    }
    return(list(tx_compat_cvg,read_counts))
}

################################################################################
###########Function to compile compatible reads as HtSeq for each gene####
## .IntersectionStrict2: function modified from GenomicAlignment package for
##                      selection of compatible read/read pairs for each genes 
##                      following HTseq.
################################################################################
## input: please refer to IntersectionStrict function from GenomicAlignments
##        In particular, ignore.strand should be set as TRUE for paired-end.
##        For single-end data, depending on the sequencing, users should
##        set it accordingly.
################################################################################
## output: 
##        reads IDs compatible with features.
################################################################################
.IntersectionStrict2= function (features, reads,
                                ignore.strand = TRUE, inter.feature = TRUE) {
    ##function adapted to compile compatible reads as HTseq

    ov <- findOverlaps(reads, features, type = "within", 
                    ignore.strand = ignore.strand)
    if (inter.feature) {
        reads_to_keep <- which(countQueryHits(ov) == 1L)
        ov <- ov[queryHits(ov) %in% reads_to_keep]
    }
    ov
}

# Generated by using Rcpp::compileAttributes() -> do not edit by hand
# Generator token: 10BE3573-1514-4C36-9D1C-5A225CD40393

.bin_drop <- function(bin_size, bin_number, res_std_max) {
    .Call(`_DegNorm_bin_drop`, bin_size, bin_number, res_std_max)
}

.NMFCPP <- function(f, loop) {
    .Call(`_DegNorm_NMFCPP`, f, loop)
}

.optiNMFCPP <- function(f, normFactor, loop) {
    .Call(`_DegNorm_optiNMFCPP`, f, normFactor, loop)
}

.optiNMFCPP_grid <- function(f, normFactor, loop, grid_size) {
    .Call(`_DegNorm_optiNMFCPP_grid`, f, normFactor, loop, grid_size)
}

## ----setup, include = FALSE---------------------------------------------------
library(DegNorm)
knitr::opts_chunk$set(
    collapse = TRUE,
    comment = "#>"
)
library(knitr)
knit_hooks$set(optipng = hook_optipng)

## ----eval=FALSE---------------------------------------------------------------
# if(!requireNamespace("BiocManager", quietly = TRUE))
#     install.packages("BiocManager")
# BiocManager::install("DegNorm")

## -----------------------------------------------------------------------------
## specify bam_files from RNA-seq, you should replace it by your own bam files
bam_file_list=list.files(path=system.file("extdata",package="DegNorm"),
                        pattern=".bam$",full.names=TRUE)

## -----------------------------------------------------------------------------
## gtf_file you used for RNA-seq alignment, replace it by your own gtf file
gtf_file=list.files(path=system.file("extdata",package="DegNorm"),
                    pattern=".gtf$",full.names=TRUE)

## ----eval=TRUE----------------------------------------------------------------
## calculate the read coverage score for all genes of all samples
coverage_res_chr21_sub=read_coverage_batch(bam_file_list, gtf_file,cores=2)

## ----eval=FALSE---------------------------------------------------------------
# ## save the coverage results
# save(coverage_res_chr21_sub,file="coverage_res_chr21_sub.Rda")

## -----------------------------------------------------------------------------
data("coverage_res_chr21")
## summarize the coverage results
summary_CoverageClass(coverage_res_chr21)

## -----------------------------------------------------------------------------
## extract coverage scores and counts from coverage_res
coverage_matrix=coverage_res_chr21$coverage
counts=coverage_res_chr21$counts

## ----eval=TRUE----------------------------------------------------------------
res_DegNorm_chr21 = degnorm(read_coverage = coverage_res_chr21[[1]],
                    counts = coverage_res_chr21[[2]],
                    iteration = 5,
                    down_sampling = 1,
                    grid_size=10,
                    loop = 100,
                    cores=2)

## ----eval=FALSE---------------------------------------------------------------
# ## save the DegNorm results
# save(res_DegNorm_chr21,file="res_DegNorm_chr21.Rda")

## -----------------------------------------------------------------------------
data("res_DegNorm_chr21")

## -----------------------------------------------------------------------------
## summary of the DegNorm output
summary_DegNormClass(res_DegNorm_chr21)

## -----------------------------------------------------------------------------
## extrac normalized read counts
counts_normed=res_DegNorm_chr21$counts_normed

## ----fig.width=5,fig.height=4,message=FALSE,eval=TRUE-------------------------
##gene named "SOD1"
plot_coverage(gene_name="SOD1", coverage_output=coverage_res_chr21, 
            degnorm_output=res_DegNorm_chr21,group=c(0,1,1))

## ----fig.width=5,fig.height=4,message=FALSE,eval=TRUE-------------------------
##gene named "SOD1"
plot_coverage(gene_name="SOD1", coverage_output=coverage_res_chr21, 
            degnorm_output=res_DegNorm_chr21,group=c(0,1), 
            samples=c("SRR873822_chr21.bam", "SRR873834_chr21.bam"))

## ----fig.width=5,fig.height=4,message=FALSE,warning=FALSE,eval=TRUE-----------
plot_boxplot(DI=res_DegNorm_chr21$DI)

## ----fig.width=5,fig.height=4,message=FALSE,eval=TRUE-------------------------
plot_heatmap(DI=res_DegNorm_chr21$DI)

## ----fig.width=5,fig.height=4,message=FALSE,warning=FALSE,eval=TRUE-----------
plot_corr(DI=res_DegNorm_chr21$DI)

## ----sessionInfo--------------------------------------------------------------
sessionInfo()

################################################################################
# summary function for CoverageClass and DegNormClass
#   - summary_CoverageClass
#   - summary_DegNormClass
# Plot functions:
#   - plot_corr(DI)
#   - plot_boxplot(DI)
#   - plot_heatmap
#   - plot_coverage(coverage)
################################################################################
summary_CoverageClass=function(object){
    cat("CoverageClass from read_coverage_batch function","\n")
    cat(paste("$",names(object)[1],sep=""),"\t:", class(object$coverage), 
        "of length", length(object$coverage),"\n")
    cat(paste("$",names(object)[2],sep=""),"\t:", class(object$counts), 
        "of dimension", dim(object$counts)[1],"by",dim(object$counts)[2],"\n")
    cat("\nSamples:\t\t",colnames(object[[2]]),"\n")
    cat("Total number genes:\t", nrow(object[[2]]),"\n")
}

summary_DegNormClass=function(object){
    cat("DegNormClass from DegNorm function:","\n")
    cat(paste("$",names(object)[1],sep=""),"\t:", class(object$counts), 
        "of dimension", dim(object$counts)[1],"by",dim(object$counts)[2],"\n")
    cat(paste("$",names(object)[2],sep=""),"\t:", class(object$counts_normed), 
        "of dimension", dim(object$counts_normed)[1],"by",
        dim(object$counts_normed)[2],"\n")
    cat(paste("$",names(object)[3],sep=""),"\t\t:", class(object$DI), 
        "of dimension", dim(object$DI)[1],"by",dim(object$DI)[2],"\n")
    cat(paste("$",names(object)[4],sep=""),"\t\t:", class(object$K), 
        "of dimension", dim(object$K)[1],"by",dim(object$K)[2],"\n")
    cat(paste("$",names(object)[5],sep=""),"\t:", class(object$convergence), 
        "of length", length(object$convergence),"\n")
    cat(paste("$",names(object)[6],sep=""),"\t:", class(object$envelop), 
        "of length", length(object$envelop),"\n")
    cat("\nSamples:\t\t",colnames(object[[2]]),"\n")
    cat("Total number genes:\t", nrow(object[[2]]),"\n")
}

################################################################################
# plot functions for heatmap, boxplot and coverage curves
################################################################################
################################################################################
# plot.coverage: function to plot before/after degradation normalization 
#               coverage curve
################################################################################
plot_coverage <- function(gene_name, coverage_output, degnorm_output,
                            group=NULL, samples=NULL){
    # coverage plot before and after normalization for one single gene
    # input:
    #    - gene_name: the name of the gene
    #    - coverage_res: the output from read_coverage_batch function
    #    - degnorm_output: the output from DegNorm function
    #    - group: integer values indicating the group idenity of samples
    #    - samples: string vector for the subset of samples to be plotted.
    #              NULL means all samples to be plotted. The length of samples
    #              must be of the same length of group if both specified.
    # output:
    #    - a ggplot2 object of coverage plot before/after DegNorm normalization
    #    - group: a vector of factors representing sample conditions in the 
    #      order of the coverage matrix columns, either integer or char.
    # retrieve raw coverage from read_coverage_batch output coverage_res
    if(!is.null(samples)&&!is.null(group)&&length(group)!=length(samples)){
        stop("Error: group and samples arguments are of different length!")}
    if(sum(samples %in% colnames(coverage_output$counts))<length(samples)){
         stop("Error: some sample names do not exist!")}
    if(length(group)>ncol(coverage_output$counts)) stop("Error: Number of
        samples in Group exceeds the number of samples in the data!")
    raw.coverage.matrix = data.table(t(coverage_output$coverage[[gene_name]]))
    n.sample = dim(raw.coverage.matrix)[2]
    # retrieve abundance scale K and envelope function from DegNorm output 
    K=as.matrix(degnorm_output$K[which(row.names(degnorm_output$K)
                                        ==gene_name),])
    envelop=degnorm_output$envelop[[gene_name]]
    degnorm.coverage.matrix = data.table(t(K%*%envelop))
    n=nrow(raw.coverage.matrix)
    raw.coverage.matrix=cbind(position=rep(seq_len(n),n.sample),label=
                        rep("raw",n.sample*n),stack(raw.coverage.matrix))
    degnorm.coverage.matrix=cbind(position=rep(seq_len(n),n.sample),
                                label=rep("DegNorm",n.sample*n),
                                stack(degnorm.coverage.matrix))
    dat.curve = data.table(rbind(raw.coverage.matrix, degnorm.coverage.matrix))
    colnames(dat.curve)[3:4] = c("coverage", "sample")
    
    if (is.null(samples)==FALSE){
        dat.curve=dat.curve[dat.curve$sample==samples]
    }
    default.palette=c("#0072B2","#D55E00","#009E73","#999999", "#E69F00", 
                        "#56B4E9", "#F0E442",  "#CC79A7")
    if(is.null(group)){ #customize color palette
        if(n.sample <= length(default.palette)){
            custom_color = default.palette[seq_len(n.sample)]
        }else{
            custom_color = plasma(n.sample)
        }
    }else{
        n.group = length(unique(group))
        if(length(n.group) <= length(default.palette)){
            custom_color = rep(default.palette[seq_len(n.group)], n.group)
        }else{
            custom_color=rep(plasma(length(n.group), end=0.9),n.group)
        }
    }
    p = ggplot(data = dat.curve, aes(x = dat.curve$position,
                            y = dat.curve$coverage,col = dat.curve$sample))+
        xlab("Transcript position") + ylab("Coverage score") +
        scale_color_manual(values = custom_color) +
        geom_line(size = 0.8) + theme_light() + facet_grid(dat.curve$label~.)
    return(p)
}

################################################################################
# generate boxplots from DI scores
################################################################################

plot_boxplot<-function(DI){
    # input:
    #     DI: a data table with each row  gene, each column for sample
    # output file:
    #     a ggplot2 object for boxplot
    
    dat_DI = data.frame(stack(DI))
    colnames(dat_DI)=c("gene_names","sample","DI_score")
    
    p.boxplot = 
        ggplot(data = dat_DI, aes(x = dat_DI$sample, y = dat_DI$DI_score)) + 
        xlab("Sample") + ylab("DI score") +
        stat_boxplot(geom ='errorbar') +
        geom_boxplot(aes(x = dat_DI$sample, y = dat_DI$DI_score, 
                        col = dat_DI$sample), outlier.shape = NA) +
        theme_classic() + ylim(c(0,max(DI))) +
        theme(legend.position="none")
    return(p.boxplot)
}

################################################################################
# generate heatmaps for the correlation matrix from DI scores
plot_corr<-function(DI){
    # input:
    #     DI: a data table with each row  gene, each column for sample
    # output file:
    #     a ggplot2 object for correlation matrix heatmap 

    p = heatmaply(apply(cor(DI),2,rev), 
        dendrogram = 'none',  # without dendrogram
        margins = c(100,100),
        width = 1200, height = 1000)
return(p)
}

################################################################################
# generate heatmaps of the DI matrix
plot_heatmap<-function(DI){
    # input:
    #     DI: a data table with each row  gene, each column for sample
    # output file:
    #     a ggplot2 object for correlation matrix heatmap 

    p1 = plot_ly(x=colnames(DI),z = as.matrix(DI[order(rowSums(DI)),]), 
            #colorscale = colorRamp(c("blue", "red")), 
            type = "heatmap", showscale = TRUE)
    # sort each column
    DI = apply(DI,2,sort)
    p2 = plot_ly(x=colnames(DI),z = as.matrix(DI[order(rowSums(DI)),]), 
            #colorscale = colorRamp(c("blue", "red")), 
            type = "heatmap", showscale = FALSE)
    # join two figures
    p = subplot(p1, p2, nrows = 1, shareY = TRUE)
return(p)
}

## ----setup, include = FALSE---------------------------------------------------
library(DegNorm)
knitr::opts_chunk$set(
    collapse = TRUE,
    comment = "#>"
)
library(knitr)
knit_hooks$set(optipng = hook_optipng)

## ----eval=FALSE---------------------------------------------------------------
#  if(!requireNamespace("BiocManager", quietly = TRUE))
#      install.packages("BiocManager")
#  BiocManager::install("DegNorm")

## -----------------------------------------------------------------------------
## specify bam_files from RNA-seq, you should replace it by your own bam files
bam_file_list=list.files(path=system.file("extdata",package="DegNorm"),
                        pattern=".bam$",full.names=TRUE)

## -----------------------------------------------------------------------------
## gtf_file you used for RNA-seq alignment, replace it by your own gtf file
gtf_file=list.files(path=system.file("extdata",package="DegNorm"),
                    pattern=".gtf$",full.names=TRUE)

## ----eval=TRUE----------------------------------------------------------------
## calculate the read coverage score for all genes of all samples
coverage_res_chr21_sub=read_coverage_batch(bam_file_list, gtf_file,cores=2)

## ----eval=FALSE---------------------------------------------------------------
#  ## save the coverage results
#  save(coverage_res_chr21_sub,file="coverage_res_chr21_sub.Rda")

## -----------------------------------------------------------------------------
data("coverage_res_chr21")
## summarize the coverage results
summary_CoverageClass(coverage_res_chr21)

## -----------------------------------------------------------------------------
## extract coverage scores and counts from coverage_res
coverage_matrix=coverage_res_chr21$coverage
counts=coverage_res_chr21$counts

## ----eval=TRUE----------------------------------------------------------------
res_DegNorm_chr21 = degnorm(read_coverage = coverage_res_chr21[[1]],
                    counts = coverage_res_chr21[[2]],
                    iteration = 5,
                    down_sampling = 1,
                    grid_size=10,
                    loop = 100,
                    cores=2)

## ----eval=FALSE---------------------------------------------------------------
#  ## save the DegNorm results
#  save(res_DegNorm_chr21,file="res_DegNorm_chr21.Rda")

## -----------------------------------------------------------------------------
data("res_DegNorm_chr21")

## -----------------------------------------------------------------------------
## summary of the DegNorm output
summary_DegNormClass(res_DegNorm_chr21)

## -----------------------------------------------------------------------------
## extrac normalized read counts
counts_normed=res_DegNorm_chr21$counts_normed

## ----fig.width=5,fig.height=4,message=FALSE,eval=TRUE-------------------------
##gene named "SOD1"
plot_coverage(gene_name="SOD1", coverage_output=coverage_res_chr21, 
            degnorm_output=res_DegNorm_chr21,group=c(0,1,1))

## ----fig.width=5,fig.height=4,message=FALSE,eval=TRUE-------------------------
##gene named "SOD1"
plot_coverage(gene_name="SOD1", coverage_output=coverage_res_chr21, 
            degnorm_output=res_DegNorm_chr21,group=c(0,1), 
            samples=c("SRR873822_chr21.bam", "SRR873834_chr21.bam"))

## ----fig.width=5,fig.height=4,message=FALSE,warning=FALSE,eval=TRUE-----------
plot_boxplot(DI=res_DegNorm_chr21$DI)

## ----fig.width=5,fig.height=4,message=FALSE,eval=TRUE-------------------------
plot_heatmap(DI=res_DegNorm_chr21$DI)

## ----fig.width=5,fig.height=4,message=FALSE,warning=FALSE,eval=TRUE-----------
plot_corr(DI=res_DegNorm_chr21$DI)

## ----sessionInfo--------------------------------------------------------------
sessionInfo()

# Generated by using Rcpp::compileAttributes() -> do not edit by hand
# Generator token: 10BE3573-1514-4C36-9D1C-5A225CD40393

.bin_drop <- function(bin_size, bin_number, res_std_max) {
    .Call(`_DegNorm_bin_drop`, bin_size, bin_number, res_std_max)
}

.NMFCPP <- function(f, loop) {
    .Call(`_DegNorm_NMFCPP`, f, loop)
}

.optiNMFCPP <- function(f, normFactor, loop, baseline) {
    .Call(`_DegNorm_optiNMFCPP`, f, normFactor, loop, baseline)
}

.optiNMFCPP_grid <- function(f, normFactor, loop, grid_size) {
    .Call(`_DegNorm_optiNMFCPP_grid`, f, normFactor, loop, grid_size)
}

---
title: "DegNorm: an R package for degradation normalization for RNA-seq data"
author: "Bin Xiong, Ji-Ping Wang"
date: "`r Sys.Date()`"
output: 
    rmarkdown::html_document:
        highlight: pygments
        toc: true
        fig_width: 5
vignette: >
    %\VignetteIndexEntry{DegNorm}
    %\VignetteEngine{knitr::rmarkdown}
    %\VignetteEncoding{UTF-8}
---


**Maintainer**: Ji-Ping Wang, <<jzwang@northwestern.edu>>

<!-- ([**Python package download**](https://nustatbioinfo.github.io/DegNorm/))**-->


```{r setup, include = FALSE}
library(DegNorm)
knitr::opts_chunk$set(
    collapse = TRUE,
    comment = "#>"
)
library(knitr)
knit_hooks$set(optipng = hook_optipng)
```

**Reference**:
Xiong, B., Yang, Y., Fineis, F. Wang, J.-P., DegNorm: normalization of 
generalized transcript degradation improves accuracy in RNA-seq analysis, 
Genome Biology, 2019,20:75

## What is DegNorm?

DegNorm, short for **Deg**radation **Norm**alization, is a bioinformatics 
pipeline designed to correct  for bias due to the heterogeneous patterns of
transcript degradation in RNA-seq data. DegNorm 
helps improve the accuracy of the differential expression analysis by 
accounting for this degradation. 

In practice, RNA samples are often more-or-less degraded, and the degradation
severity is not only sample-specific, but gene-specific as well. It is known 
that longer genes tend to degrade faster  than shorter ones. As such, 
commonplace global degradation normalization approaches that impose a 
single normalization factor on all genes within a sample can be ineffective
in correcting for RNA degradation bias.


## DegNorm pipline available formats

We've developed an R package and an indepedent Python package
([download](https://nustatbioinfo.github.io/DegNorm/)), 
both of which allow to run the entire pipeline from the RNA-seq alignment 
(.bam) files. **For most-updated version, we recommend to use R package from bioconductor**.

## DegNorm version 1.3.4 updates

In version 1.3.1, we made following updates:
1. In `plot_coverage` funciton, an `samples` options is provided to allow user to select which
samples to plot for coverage curves.
2. We fixed a bug in `DegNorm` funciton. In earlier version, we used genes with maximum `rho<0.1` from initial SVD to determine the scaling factor before running core algorithm. For some data with many samples and large degradation, it may return `NA` to cause issues. 


## Install DegNorm R package

```{r,eval=FALSE}
if(!requireNamespace("BiocManager", quietly = TRUE))
    install.packages("BiocManager")
BiocManager::install("DegNorm")
```

## DegNorm main features

DegNorm R package contains two major functions: (1) processing the RNA-seq 
alignment file (.bam) to calculate the coverage; and (2) using a core algorithm 
written in RcppArmadillo to perform rank-one over-approximation on converage 
matrices for each  gene to estimate the degramation index (DI) score for each 
gene within each sample.

DegNorm outputs DI scores together with degradation-normalized read counts 
(based on DI scores).  It also provides supplementary functions for 
visualization of degradation at both gene and sample level. The following 
diagram illustrates the flow of DegNorm pipeline.  

&nbsp;



![A diagram of DegNorm.](degnorm_logo.png){width=60%}  


&nbsp;

The following vignette is intended to provide example codes for running 
DegNorm R package. It presumes that you have successfully installed 
DegNorm package. We illustrate below how to: 1) calculate the read coverage 
curves for all genes within  all samples, and 2) perform degradation 
normalization on coverage curves.  Either step is computing intensive. 
Dependent upon the number of samples and the  sequencing depth, the total 
computing time may last a few hours. DegNorm utilizes 
the parallel computing functionality of R and automatically detects the number 
of  cores on your computer to run jobs in parallel. Due to the large size 
of bam file and  limited computing power of personal computer, we recommend
users to  run it in servers or computing clusters.



### 1. Compute coverage score based on alignment .bam files

##### Set up input file: .bam and .gtf files.

```{r}
## specify bam_files from RNA-seq, you should replace it by your own bam files
bam_file_list=list.files(path=system.file("extdata",package="DegNorm"),
                        pattern=".bam$",full.names=TRUE)
```
The three bam files were subsetted from a specific region of chorosome 21 from 
the origianl bam  for package size limitation. Original files can be found from
the included reference above.

```{r}
## gtf_file you used for RNA-seq alignment, replace it by your own gtf file
gtf_file=list.files(path=system.file("extdata",package="DegNorm"),
                    pattern=".gtf$",full.names=TRUE)
```

##### Run main function to create read coverage matrix and read counts

```{r,eval=TRUE}
## calculate the read coverage score for all genes of all samples
coverage_res_chr21_sub=read_coverage_batch(bam_file_list, gtf_file,cores=2)
```
`cores` argument specifies the number of cores to use. Users should
try to use as many as possible cores to maximize the computing efficiency.


```{r,eval=FALSE}
## save the coverage results
save(coverage_res_chr21_sub,file="coverage_res_chr21_sub.Rda")
```

Function `read_coverage_batch` returns the coverage matrices as a list, 
one per gene, and a dataframe for read counts, each row for one gene and 
each column for one sample.

```{r}
data("coverage_res_chr21")
## summarize the coverage results
summary_CoverageClass(coverage_res_chr21)
```

```{r}
## extract coverage scores and counts from coverage_res
coverage_matrix=coverage_res_chr21$coverage
counts=coverage_res_chr21$counts
```

### 2. DegNorm core algorithm

Run degnorm core algorithm for degradation normalization. DegNorm purpose 
is for differential expression analysis. Thus genes with extremely low read 
counts from all samples are filtered out. The current filtering criterion is
that if more than half of the samples have less than 5 read count, that gene 
will not be considered in the degnorm algorithm. In the following example, I 
am using downsamling to save time below (default). Alternatively you can set
down_sampling = 0, which takes longer time.
If `down_samplin= 1`, read coverage scores are binned with size by `grid_size` 
for baseline selection to achieve better efficiency. The default `grid_size` is
10 bp. We recommend to use a `grid_size` less than 50 bp. `iteration` specifies
the big loop in DegNorm algorithm and 5 is usually sufficient. `loop` specifies
the iteration number in the matrix factorization over-approximation.

```{r,eval=TRUE}
res_DegNorm_chr21 = degnorm(read_coverage = coverage_res_chr21[[1]],
                    counts = coverage_res_chr21[[2]],
                    iteration = 5,
                    down_sampling = 1,
                    grid_size=10,
                    loop = 100,
                    cores=2)
```
If `down_sampling= 0`, then the argument `grid_size` is ignored.


```{r,eval=FALSE}
## save the DegNorm results
save(res_DegNorm_chr21,file="res_DegNorm_chr21.Rda")
```

Function `degnorm` returns a list of multiple objects. counts_normed is the 
one with degradation normalized read counts for you to input DeSeq or EdgeR
for DE analysis.

```{r}
data("res_DegNorm_chr21")
```

```{r}
## summary of the DegNorm output
summary_DegNormClass(res_DegNorm_chr21)
```

The difference of number of genes between `res_DegNorm` and `coverage_res`
is 207 (339-132). The 207 genes were filtered out from `degnorm` degradation
normalization because less than half of the samples (3) have more than 5 
read count.


```{r}
## extrac normalized read counts
counts_normed=res_DegNorm_chr21$counts_normed
```

### 3. Plot functions in DegNorm

DegNorm provides four plot functions for visualization of degradation and
sample quality diagnosis.

* plot_coverage
* plot_corr
* plot_heatmap
* plot_boxplot

&nbsp;

##### -- Plot the before-/after-degradation coverage curves 

```{r,fig.width=5,fig.height=4,message=FALSE,eval=TRUE}
##gene named "SOD1"
plot_coverage(gene_name="SOD1", coverage_output=coverage_res_chr21, 
            degnorm_output=res_DegNorm_chr21,group=c(0,1,1))
```

In version 1.3.1, a new argument $samples$ was added so the user
can specifiy which samples to plot. 

```{r,fig.width=5,fig.height=4,message=FALSE,eval=TRUE}
##gene named "SOD1"
plot_coverage(gene_name="SOD1", coverage_output=coverage_res_chr21, 
            degnorm_output=res_DegNorm_chr21,group=c(0,1), 
            samples=c("SRR873822_chr21.bam", "SRR873834_chr21.bam"))
```
&nbsp;

##### -- Boxplot of the degradation index(DI) scores

```{r,fig.width=5,fig.height=4,message=FALSE,warning=FALSE,eval=TRUE}
plot_boxplot(DI=res_DegNorm_chr21$DI)
```
&nbsp;

##### -- Heatmap plot of the degradation index(DI) scores

```{r,fig.width=5,fig.height=4,message=FALSE,eval=TRUE}
plot_heatmap(DI=res_DegNorm_chr21$DI)
```
&nbsp;

##### -- Correlation matrix plot of degradation index(DI) scores

```{r,fig.width=5,fig.height=4,message=FALSE,warning=FALSE,eval=TRUE}
plot_corr(DI=res_DegNorm_chr21$DI)
```
&nbsp;

## Session info

```{r sessionInfo}
sessionInfo()
```

\name{DegNorm-plot-functions}
\alias{plot_corr}
\alias{plot_heatmap}
\alias{plot_boxplot}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{
Degradation index (DI) score plot functions
}
\description{
DegNorm provides three functions for visualization gene-/sample-wise 
degradation.
}
\usage{
plot_corr(DI)
plot_heatmap(DI)
plot_boxplot(DI)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
\item{DI}{a matrix or data.frame of degradation index (DI) scores with 
each row corresponding to one gene and each column for a sample.}
}
\value{
These functions return a boxplot of DI scores by sample, a heatmap 
of DIS scores of all genes in all samples and a correlation plot of 
DI scores between samples respectively.
}

\details{
\code{plot_corr}{ plots the correlation matrix of DI scores between samples.}
\code{plot_heatmap}{ plots the heatmap of DI  scores. Left is ploted in 
descending order of average DI scores of genes where each row corresponds 
to one gene. In the right plot, DI scores were sorted within each sample and 
plotted in descending order.}
\code{plot_boxplot}{plots the boxplot of DI scores by samples.}
}

\examples{
## res_DegNorm_chr21 is degnorm otuput stored in sysdata.Rda
data(res_DegNorm_chr21)
plot_boxplot(res_DegNorm_chr21$DI)
plot_heatmap(res_DegNorm_chr21$DI)
plot_corr(res_DegNorm_chr21$DI)
}

\name{res_DegNorm_chr21}
\alias{res_DegNorm_chr21}
\docType{data}
\title{
Example DegNormClass data 
}
\description{
Example of \code{DegNormClass} data  from DegNorm package. It is the 
output from \code{degnorm} function for human chromosome 21. 
}
\usage{data("res_DegNorm_chr21")}
\format{
A \code{DegNormClass} list of the following items:
\describe{
\item{\code{counts}}{a data.drame of read counts for each gene within 
each sample.}
\item{\code{counts_normed}}{a data.drame of degradation-normalized read 
counts for each gene within each sample.}
\item{\code{DI}}{a matrix of degradation index scores for each gene within 
each sample.}
\item{\code{K}}{normalizing scale factor for each gene within each sample 
after accounting for degradation normalization.}
\item{\code{convergence}}{convergence tag; 0 = degnorm was not done on this 
gene because smaller counts or too short length.1 = degnorm was done with 
baseline selection. 2 = degnorm done without baseline selection because 
gene length (after filtering out low count regions)<200 bp. 3= baseline 
was found, but DI score is too large. 4 = baseline selection didn't 
coverge.}
\item{\code{envelop}}{a list of the envelop curves for all genes.}
}}

\examples{
data(res_DegNorm_chr21)
summary_DegNormClass(res_DegNorm_chr21)
}
\keyword{datasets}

\name{coverage_res_chr21}
\alias{coverage_res_chr21}
\docType{data}
\title{
Example CoverageClass data 
}
\description{
Example of \code{CoverageClass} data  from DegNorm package. 
It is the output from \code{read_coverage_batch} function for human 
chromosome 21. 
}
\usage{data(coverage_res_chr21)}

\format{
A \code{coverageClass} list of the following
\describe{
    \item{\code{coverage}}{a list of converage matrices for all 
    genes within each sample}
    \item{\code{counts}}{a data.frame of read counts for all genes 
    within each sample.}
}
}

\examples{
data(coverage_res_chr21)
summary_CoverageClass(coverage_res_chr21)
}
\keyword{datasets}

\name{plot_coverage}
\alias{plot_coverage}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{
Coverage plot functions for DegNorm
}
\description{
\code{plot_coverage} plots the before- and after-degradation coverage curves}
\usage{
plot_coverage(gene_name, coverage_output, degnorm_output, group=NULL, samples=NULL)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
\item{gene_name}{the name of the gene whose coverage coverage to be plotted.}
\item{coverage_output}{CoverageClass object, the output from function 
\code{coverage_cal_batch}.}
\item{degnorm_output}{DegNormClass object, the output from function 
\code{DegNorm}.}
\item{group}{a vector of integers or character strings indicating the 
biological conditions of the samples. Coverage curves will be plotted
in the same color for the same group. Default is NULL. By default all
curves will plotted in different colors.}
\item{samples}{a string vector for the subset of samples to be plotted.
NULL means all samples to be plotted. The length of samples
must be of the same length of group if both specified.}
}
\value{
The coverage curve before and after degradation normalization.
}
\details{
\code{plot_coverage}{outputs the coverage curves before- and after-degradation 
normalization.}
}


\examples{
## gene named "SOD1",plot coverage curves
data(coverage_res_chr21)
data(res_DegNorm_chr21)
plot_coverage(gene_name="SOD1", coverage_output=coverage_res_chr21, 
degnorm_output=res_DegNorm_chr21, group=c(0,1,1))

}

\name{read_coverage}
\alias{read_coverage}
\title{
Function to calculate read coverage score for one bam file
}
\description{
This function judges whether bam file is single-end and paired-end,
and generate bam file index if needed. It calls function 
\code{paired_end_cov_by_ch} or \code{single_end_by_ch}.      
It takes multiple-core structure
for parallel computing for efficiency.}
\usage{
read_coverage(bam_file,all_genes,cores)
}
\arguments{
\item{bam_file}{The name of the bam file.}
\item{all_genes}{An GRangesList object. It's the parsed genes annotation 
file from GTF file.}
\item{cores}{number of cores to use.}
}
\details{
This function judges whether bam file is single-end and paired-end,
and generate bam file index if needed. It takes multiple-core structure
for parallel computing for efficiency.}
\value{
This function returns a coverageClass object.  It contains a list of: 
(1) a list of coverage score for each gene in RLE format and 
(2) a dataframe for read counts   
}

\seealso{
\code{\link[DegNorm]{read_coverage_batch}}
}
\keyword{internal}

\name{DegNorm-package}
\alias{DegNorm-package}
\docType{package}
\title{DegNorm: degradation normalization for RNA-seq data}
\description{\code{DegNorm} is an R package for degradation normalication 
for bulk RNA-seq data.DegNorm, short for degradation normalization, is a 
bioinformatics pipeline  designed to correct for bias due to the heterogeneous 
patterns of transcript degradation in RNA-seq data. 
}
\details{
DegNorm is a data-driven approach for RNA-Seq normalization resulting in 
the adjusted read count matrix. This adjustment applies to each gene within 
each sample, accounting for sample- and gene-specific degradation bias while 
simultaneously controlling for the sequencing depth. The algorithm at the 
center of DegNorm is the rank-one over-approximation of a gene's coverage 
score matrix, which is comprised of the different samples' coverage score 
curves along the transcript for each gene. For each gene, DegNorm estimates 
(1) an envelope function representing the ideal shape of the gene's coverage
curve when no degradation is present, and (2) scale factors for each sample
(for said gene) that indicates the relative abundance of the gene within the 
sample.

functions:
\code{read_coverage_batch,
degnorm,
plot_coverage,
plot_heatmap,
plot_corr,
plot_boxplot}
}
\author{
Bin Xiong, Ji-Ping Wang

Maintainer: Ji-Ping Wang <jzwang@northwestern.edu>
}
\references{
DegNorm reference: 

Xiong, B., Yang, Y., Fineis, F. Wang, J.-P., DegNorm: normalization of 
generalized transcript degradation improves accuracy in RNA-seq analysis,
Genome Biology, 2019,20:75
}
\keyword{ RNA-seq, degradation, normalization }

\name{summary_DegNormClass}
%- Also NEED an '\alias' for EACH other topic documented here.
\alias{summary_DegNormClass}
\title{
Summary method for DegNormClass.
}
\description{
It prints a summary of the data objects contained in the list from degnorm 
function.
}
\usage{
summary_DegNormClass(object)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
\item{object}{DegNormClass from \code{degnorm} function.
}
}
\value{On-screen summary of \code{DegNormClass} object.}

\examples{
## Summary of degnorm output (DegNormlass)
data(res_DegNorm_chr21)
summary_DegNormClass(res_DegNorm_chr21)
}

\name{read_coverage_batch}
\alias{read_coverage_batch}
\title{
Compute the read coverage score and read counts for all genes in batch mode
}
\description{
This function calls \code{read_coverage} to compute read coverage score and 
read counts for all genes and samples.

Notes: 
\enumerate{
  \item Coverage score is calculated per gene, i.e., concatenation of all
  exons from the same gene.
  \item We follow the HTseq protocol for counting valid reads or read pairs for each 
  gene.
  \item When reading the alignment file, \code{isSecondaryAlignment} is set to 
  \code{FALSE} to avoid redundant counting.
  \item For paired-end data, \code{isPaired} is set to \code{TRUE}. We do not recommend
  setting \code{isProperPair} to \code{TRUE} as some fragment lengths may exceed 200 bp.
  \item Users can modify \code{scanBamParam} in the R code as needed.
}
}
\usage{
read_coverage_batch(bam_file_list, gtf_file, cores = 1)
}
\arguments{
\item{bam_file_list}{A character vector of BAM file paths.}
\item{gtf_file}{The GTF file that RNA-seq reads were aligned to.}
\item{cores}{Number of processor cores to use. Default = 1.}
}
\value{
A list with the following components:
\item{coverage}{A list of coverage matrices for all genes within each sample.}
\item{counts}{A \code{data.frame} of read counts for all genes within each sample.}
}
\examples{
## Read BAM file and GTF file from the package
bam_file_list <- list.files(
    path = system.file("extdata", package = "DegNorm"),
    pattern = ".bam$", full.names = TRUE
)
gtf_file <- list.files(
    path = system.file("extdata", package = "DegNorm"),
    pattern = ".gtf$", full.names = TRUE
)

## Run read_coverage_batch to calculate read coverage curves and read counts
coverage_res <- read_coverage_batch(bam_file_list, gtf_file, cores = 2)
}
\seealso{
\code{\link[DegNorm]{read_coverage}}
}

\name{summary_CoverageClass}
%- Also NEED an '\alias' for EACH other topic documented here.
\alias{summary_CoverageClass}
\title{
Summary method for CoverageClass.
}
\description{
It prints a summary of the data objects contained in the list from 
\code{read_coverage_batch}.
}
\usage{
summary_CoverageClass(object)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
\item{object}{CoverageClass from code{read_coverage_batch.}
}}
\value{On-screen plot of summary of \code{CoverageClass} object.}
\examples{
## Summary of coverage_cal_batch output (CoverageClass)
data(coverage_res_chr21)
summary_CoverageClass(coverage_res_chr21)
}
% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.

\name{degnorm}
\alias{degnorm}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{
Main function to perform degradation normalization.
}
\description{
\code{degnorm} calcualtes the degradation index score for each 
gene within each sample and return the degradation-normalized read counts.
}
\usage{
degnorm(read_coverage,counts,iteration,loop,down_sampling=1,grid_size=10,
cores=1)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
    \item{read_coverage}{a list of converage matrices, one per gene}
    \item{counts}{dataframe of read counts, each row for one gene, and column
    for sample. The order and number of genes must match the order in
    read_coverage matrices.}
    \item{iteration}{iteration number for degnorm algorithm. 5 is sufficient.}
    \item{loop}{iteration number inside of nonnegative matrix 
    factorization-over approximation. Default is 100.}
    \item{down_sampling}{1 for yes (default) and 0 for no. If yes, average
    coverage score is calcualted on a grid of size specified by grid_size 
    argument. The new coverage matrix formed by the grid average score will 
    be used for baseline selection. This increases the efficiency of algorithm 
    while maintaining comparable accuracy.}
    \item{grid_size}{default size is 10 bp.}
    \item{cores}{number of cores. Default number if 1. Users should input 
    the maximum possible number of cores for efficiency.}
}

\value{
\code{degnorm} outputs a list of following objects:
\item{counts}{a data.drame of read counts for each gene within each sample.}
\item{counts_normed}{a data.drame of degradation-normalized read counts for
each gene within each sample.}
\item{DI}{a matrix of degradation index scores for each gene within each 
sample.}
\item{K}{normalizing scale factor for each gene within each sample after
accounting for degradation normalization.}
\item{convergence}{convergence tag; 0 = degnorm was not done on this gene 
because smaller counts or too short length.1 = degnorm was done with baseline 
selection. 2 = degnorm done without baseline selection because gene length 
(after filtering out low count regions)<200 bp. 3= baseline was found, but 
DI score is too large. 4 = baseline selection didn't coverge. }
\item{envelop}{list of the envelop curves for all genes.}
}


\examples{
##coverage_res_chr21 is a \code{CoverageClass} object from DegNorm Package.
data(coverage_res_chr21)
res_DegNorm = degnorm(read_coverage = coverage_res_chr21[[1]],
                    counts = coverage_res_chr21[[2]],
                    iteration = 2,
                    down_sampling = 1,
                    grid_size=10,
                    loop = 20,
                    cores=2)
}
% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
\keyword{ ~kwd1 }% use one of  RShowDoc("KEYWORDS")
\keyword{ ~kwd2 }% __ONLY ONE__ keyword per line
